==> verttice_ui-0.0.34.tar.gz: lib/src/widgets/drawing/painter.dart <==
import 'dart:math';
import 'dart:typed_data';
import 'package:verttice_logic/verttice_logic.dart';
import 'package:verttice_ui/src/config/config.dart';
import 'package:verttice_ui/src/const/box_sizes.dart';
import 'package:verttice_ui/src/const/icons.dart';
import 'package:flutter/material.dart';
import 'package:scribble/scribble.dart';
import 'package:flutter_state_notifier/flutter_state_notifier.dart';
import 'package:verttice_ui/src/widgets/dialogs/full_page_dialog.dart';
import 'package:verttice_ui/src/widgets/themed_app_bar.dart';
import 'package:verttice_ui/src/widgets/themed_progress.dart';

class SaveImage extends StatefulWidget {
  final Function(FileModel data) imageDataCallback;
  final String path;
  final Uint8List image;
  const SaveImage({
    Key? key,
    required this.image,
    required this.imageDataCallback,
    required this.path,
  }) : super(key: key);

  @override
  State<SaveImage> createState() => _SaveImageState();
}

class _SaveImageState extends State<SaveImage> {
  bool _isUploadingImage = false;
  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: ThemedAppBar(
        actions: [
          if (_isUploadingImage)
            const Padding(
              padding: EdgeInsets.all(8.0),
              child: Center(child: ThemedProgress()),
            ),
          sizedBoxW8,
          Tooltip(
            message: 'Save image',
            child: IconButton(
                onPressed: () async {
                  setState(() {
                    _isUploadingImage = true;
                  });
                  final String fileID = IDs.l16();
                  final response = await VertticeUIConfig.instance
                      .uploadFiles(context: context, files: [
                    UploadFileDataModel(
                      fileID: fileID,
                      file: widget.image,
                      mimeType: "image/png",
                      path: "${widget.path}/$fileID",
                      name: fileID,
                      size: widget.image.length,
                    ),
                  ]);

                  setState(() {
                    _isUploadingImage = false;
                  });
                  if (response != null) {
                    final newFile = response.first;

                    widget.imageDataCallback(newFile);
                  }

                  Navigator.pop(context);
                },
                icon: ThemedIcons.check()),
          )
        ],
      ),
      body: Image.memory(widget.image),
    );
  }
}

class Painter extends StatefulWidget {
  final Function(FileModel data) imageDataCallback;
  final String path;
  final String? buttonText;

  const Painter({
    Key? key,
    required this.imageDataCallback,
    required this.path,
    this.buttonText,
  }) : super(key: key);

  @override
  State<Painter> createState() => _PainterState();
}

class _PainterState extends State<Painter> {
  late ScribbleNotifier notifier;

  @override
  void initState() {
    notifier = ScribbleNotifier();
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    final height = MediaQuery.of(context).size.height;
    final width = MediaQuery.of(context).size.width;
    return Scaffold(
      appBar: ThemedAppBar(
        title: "Paint",
        actions: [
          IconButton(
            icon: const Icon(Icons.save),
            tooltip: "Save as image",
            onPressed: () => _saveImage(context),
          )
        ],
      ),
      body: Column(
        children: [
          SizedBox(
            height: 70,
            child: ListView(
              scrollDirection: Axis.horizontal,
              children: [
                sizedBoxW16,
                _buildColorToolbar(context),
                const Divider(
                  height: 32,
                ),
                _buildStrokeToolbar(context),
              ],
            ),
          ),
          Expanded(
            child: ConstrainedBox(
              constraints: BoxConstraints(
                maxHeight: min(height, 500),
                maxWidth: min(width - 40, 600),
              ),
              child: Center(
                child: AspectRatio(
                  aspectRatio: 16 / 9,
                  child: Container(
                    decoration: BoxDecoration(
                      image: const DecorationImage(
                        fit: BoxFit.cover,
                        image: AssetImage('signatureBackground.jpg'),
                      ),
                      color: Colors.white,
                      border:
                          Border.all(width: 1, color: Colors.blueGrey[700]!),
                      borderRadius: const BorderRadius.all(
                        Radius.circular(7.0),
                      ),
                    ),
                    child: Scribble(
                      minWidthFactor: 0.2,
                      notifier: notifier,
                      drawPen: true,
                    ),
                  ),
                ),
              ),
            ),
          )
        ],
      ),
    );
  }

  Future<void> _saveImage(BuildContext context) async {
    final image = await notifier.renderImage();

    await fullPageDialog(
      context: context,
      builder: (BuildContext context) => SaveImage(
        image: image.buffer.asUint8List(),
        imageDataCallback: widget.imageDataCallback,
        path: widget.path,
      ),
    );
  }

  Widget _buildStrokeToolbar(BuildContext context) {
    return StateNotifierBuilder<ScribbleState>(
      stateNotifier: notifier,
      builder: (context, state, _) => Row(
        crossAxisAlignment: CrossAxisAlignment.center,
        mainAxisAlignment: MainAxisAlignment.start,
        children: [
          for (final w in notifier.widths)
            _buildStrokeButton(
              context,
              strokeWidth: w,
              state: state,
            ),
        ],
      ),
    );
  }

  Widget _buildStrokeButton(
    BuildContext context, {
    required double strokeWidth,
    required ScribbleState state,
  }) {
    final selected = state.selectedWidth == strokeWidth;
    return Padding(
      padding: const EdgeInsets.all(4),
      child: Material(
        elevation: selected ? 4 : 0,
        shape: const CircleBorder(),
        child: InkWell(
          onTap: () => notifier.setStrokeWidth(strokeWidth),
          customBorder: const CircleBorder(),
          child: AnimatedContainer(
            duration: kThemeAnimationDuration,
            width: strokeWidth * 2,
            height: strokeWidth * 2,
            decoration: BoxDecoration(
                color: state.map(
                  drawing: (s) => Color(s.selectedColor),
                  erasing: (_) => Colors.transparent,
                ),
                border: state.map(
                  drawing: (_) => null,
                  erasing: (_) => Border.all(width: 1),
                ),
                borderRadius: BorderRadius.circular(50.0)),
          ),
        ),
      ),
    );
  }

  Widget _buildColorToolbar(BuildContext context) {
    return StateNotifierBuilder<ScribbleState>(
      stateNotifier: notifier,
      builder: (context, state, _) => Row(
        crossAxisAlignment: CrossAxisAlignment.center,
        mainAxisAlignment: MainAxisAlignment.start,
        children: [
          _buildUndoButton(context),
          sizedBoxW4,
          _buildRedoButton(context),
          sizedBoxW4,
          _buildClearButton(context),
          sizedBoxW4,
          _buildPointerModeSwitcher(context,
              penMode:
                  state.allowedPointersMode == ScribblePointerMode.penOnly),
          sizedBoxW16,
          _buildEraserButton(context, isSelected: state is Erasing),
          _buildColorButton(context, color: Colors.black, state: state),
          _buildColorButton(context, color: Colors.red, state: state),
          _buildColorButton(context, color: Colors.green, state: state),
          _buildColorButton(context, color: Colors.blue, state: state),
          _buildColorButton(context, color: Colors.yellow, state: state),
        ],
      ),
    );
  }

  Widget _buildPointerModeSwitcher(BuildContext context,
      {required bool penMode}) {
    return FloatingActionButton.small(
      onPressed: () => notifier.setAllowedPointersMode(
        penMode ? ScribblePointerMode.all : ScribblePointerMode.penOnly,
      ),
      tooltip:
          "Switch drawing mode to " + (penMode ? "all pointers" : "pen only"),
      child: AnimatedSwitcher(
        duration: kThemeAnimationDuration,
        child: !penMode
            ? const Icon(
                Icons.touch_app,
                key: ValueKey(true),
              )
            : const Icon(
                Icons.do_not_touch,
                key: ValueKey(false),
              ),
      ),
    );
  }

  Widget _buildEraserButton(BuildContext context, {required bool isSelected}) {
    return Padding(
      padding: const EdgeInsets.all(4),
      child: FloatingActionButton.small(
        tooltip: "Erase",
        backgroundColor: const Color(0xFFF7FBFF),
        elevation: isSelected ? 10 : 2,
        shape: !isSelected
            ? const CircleBorder()
            : RoundedRectangleBorder(
                borderRadius: BorderRadius.circular(8),
              ),
        child: const Icon(Icons.remove, color: Colors.blueGrey),
        onPressed: notifier.setEraser,
      ),
    );
  }

  Widget _buildColorButton(
    BuildContext context, {
    required Color color,
    required ScribbleState state,
  }) {
    final isSelected = state is Drawing && state.selectedColor == color.value;
    return Padding(
      padding: const EdgeInsets.all(4),
      child: FloatingActionButton.small(
          backgroundColor: color,
          elevation: isSelected ? 10 : 2,
          shape: !isSelected
              ? const CircleBorder()
              : RoundedRectangleBorder(
                  borderRadius: BorderRadius.circular(8),
                ),
          child: Container(),
          onPressed: () => notifier.setColor(color)),
    );
  }

  Widget _buildUndoButton(
    BuildContext context,
  ) {
    return FloatingActionButton.small(
      tooltip: "Undo",
      onPressed: notifier.canUndo ? notifier.undo : null,
      disabledElevation: 0,
      backgroundColor: notifier.canUndo ? Colors.blueGrey : Colors.grey,
      child: const Icon(
        Icons.undo_rounded,
        color: Colors.white,
      ),
    );
  }

  Widget _buildRedoButton(
    BuildContext context,
  ) {
    return FloatingActionButton.small(
      tooltip: "Redo",
      onPressed: notifier.canRedo ? notifier.redo : null,
      disabledElevation: 0,
      backgroundColor: notifier.canRedo ? Colors.blueGrey : Colors.grey,
      child: const Icon(
        Icons.redo_rounded,
        color: Colors.white,
      ),
    );
  }

  Widget _buildClearButton(BuildContext context) {
    return FloatingActionButton.small(
      tooltip: "Clear",
      onPressed: notifier.clear,
      disabledElevation: 0,
      backgroundColor: Colors.blueGrey,
      child: const Icon(Icons.clear),
    );
  }
}

==> verttice_ui-0.0.97.tar.gz: lib/src/widgets/drawing/painter.dart <==
import 'dart:math';
import 'dart:typed_data';
import 'package:verttice_logic/verttice_logic.dart';
import 'package:flutter/material.dart';
import 'package:scribble/scribble.dart';
import 'package:flutter_state_notifier/flutter_state_notifier.dart';
import 'package:verttice_ui/verttice_ui.dart';

class SaveImage extends StatefulWidget {
  final Function(FileModel data) imageDataCallback;
  final String path;
  final Uint8List image;
  const SaveImage({
    Key? key,
    required this.image,
    required this.imageDataCallback,
    required this.path,
  }) : super(key: key);

  @override
  State<SaveImage> createState() => _SaveImageState();
}

class _SaveImageState extends State<SaveImage> {
  bool _isUploadingImage = false;
  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: ThemedAppBar(
        actions: [
          if (_isUploadingImage)
            const Padding(
              padding: EdgeInsets.all(8.0),
              child: Center(child: ThemedProgress()),
            ),
          sizedBoxW8,
          Tooltip(
            message: 'Save image',
            child: IconButton(
                onPressed: () async {
                  setState(() {
                    _isUploadingImage = true;
                  });
                  final String fileID = IDs.l16();
                  final response = await VertticeUIConfig.instance
                      .uploadFiles(context: context, files: [
                    UploadFileDataModel(
                      fileID: fileID,
                      file: widget.image,
                      mimeType: "image/png",
                      path: "${widget.path}/$fileID",
                      name: fileID,
                      size: widget.image.length,
                    ),
                  ]);

                  setState(() {
                    _isUploadingImage = false;
                  });
                  if (response.isNotEmpty) {
                    final newFile = response.first;

                    widget.imageDataCallback(newFile);
                  }

                  Navigator.pop(context);
                },
                icon: ThemedIcons.check()),
          )
        ],
      ),
      body: Image.memory(widget.image),
    );
  }
}

class Painter extends StatefulWidget {
  final Function(FileModel data) imageDataCallback;
  final String path;
  final String? buttonText;

  const Painter({
    Key? key,
    required this.imageDataCallback,
    required this.path,
    this.buttonText,
  }) : super(key: key);

  @override
  State<Painter> createState() => _PainterState();
}

class _PainterState extends State<Painter> {
  late ScribbleNotifier notifier;

  @override
  void initState() {
    notifier = ScribbleNotifier();
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    final height = MediaQuery.of(context).size.height;
    final width = MediaQuery.of(context).size.width;
    return Scaffold(
      appBar: ThemedAppBar(
        title: "Paint",
        actions: [
          IconButton(
            icon: const Icon(Icons.save),
            tooltip: "Save as image",
            onPressed: () => _saveImage(context),
          )
        ],
      ),
      body: Column(
        children: [
          SizedBox(
            height: 70,
            child: ListView(
              scrollDirection: Axis.horizontal,
              children: [
                sizedBoxW16,
                _buildColorToolbar(context),
                const Divider(
                  height: 32,
                ),
                _buildStrokeToolbar(context),
              ],
            ),
          ),
          Expanded(
            child: ConstrainedBox(
              constraints: BoxConstraints(
                maxHeight: min(height, 500),
                maxWidth: min(width - 40, 600),
              ),
              child: Center(
                child: AspectRatio(
                  aspectRatio: 16 / 9,
                  child: Container(
                    decoration: BoxDecoration(
                      image: const DecorationImage(
                        fit: BoxFit.cover,
                        image: AssetImage(
                          VertticeUIAssets.signatureBackground,
                          package: 'verttice_ui',
                        ),
                      ),
                      color: Colors.white,
                      border:
                          Border.all(width: 1, color: Colors.blueGrey[700]!),
                      borderRadius: const BorderRadius.all(
                        Radius.circular(7.0),
                      ),
                    ),
                    child: Scribble(
                      minWidthFactor: 0.2,
                      notifier: notifier,
                      drawPen: true,
                    ),
                  ),
                ),
              ),
            ),
          )
        ],
      ),
    );
  }

  Future<void> _saveImage(BuildContext context) async {
    final image = await notifier.renderImage();

    await fullPageDialog(
      context: context,
      builder: (BuildContext context) => SaveImage(
        image: image.buffer.asUint8List(),
        imageDataCallback: widget.imageDataCallback,
        path: widget.path,
      ),
    );
  }

  Widget _buildStrokeToolbar(BuildContext context) {
    return StateNotifierBuilder<ScribbleState>(
      stateNotifier: notifier,
      builder: (context, state, _) => Row(
        crossAxisAlignment: CrossAxisAlignment.center,
        mainAxisAlignment: MainAxisAlignment.start,
        children: [
          for (final w in notifier.widths)
            _buildStrokeButton(
              context,
              strokeWidth: w,
              state: state,
            ),
        ],
      ),
    );
  }

  Widget _buildStrokeButton(
    BuildContext context, {
    required double strokeWidth,
    required ScribbleState state,
  }) {
    final selected = state.selectedWidth == strokeWidth;
    return Padding(
      padding: const EdgeInsets.all(4),
      child: Material(
        elevation: selected ? 4 : 0,
        shape: const CircleBorder(),
        child: InkWell(
          onTap: () => notifier.setStrokeWidth(strokeWidth),
          customBorder: const CircleBorder(),
          child: AnimatedContainer(
            duration: kThemeAnimationDuration,
            width: strokeWidth * 2,
            height: strokeWidth * 2,
            decoration: BoxDecoration(
                color: state.map(
                  drawing: (s) => Color(s.selectedColor),
                  erasing: (_) => Colors.transparent,
                ),
                border: state.map(
                  drawing: (_) => null,
                  erasing: (_) => Border.all(width: 1),
                ),
                borderRadius: BorderRadius.circular(50.0)),
          ),
        ),
      ),
    );
  }

  Widget _buildColorToolbar(BuildContext context) {
    return StateNotifierBuilder<ScribbleState>(
      stateNotifier: notifier,
      builder: (context, state, _) => Row(
        crossAxisAlignment: CrossAxisAlignment.center,
        mainAxisAlignment: MainAxisAlignment.start,
        children: [
          _buildUndoButton(context),
          sizedBoxW4,
          _buildRedoButton(context),
          sizedBoxW4,
          _buildClearButton(context),
          sizedBoxW4,
          _buildPointerModeSwitcher(context,
              penMode:
                  state.allowedPointersMode == ScribblePointerMode.penOnly),
          sizedBoxW16,
          _buildEraserButton(context, isSelected: state is Erasing),
          _buildColorButton(context, color: Colors.black, state: state),
          _buildColorButton(context, color: Colors.red, state: state),
          _buildColorButton(context, color: Colors.green, state: state),
          _buildColorButton(context, color: Colors.blue, state: state),
          _buildColorButton(context, color: Colors.yellow, state: state),
        ],
      ),
    );
  }

  Widget _buildPointerModeSwitcher(BuildContext context,
      {required bool penMode}) {
    return FloatingActionButton.small(
      onPressed: () => notifier.setAllowedPointersMode(
        penMode ? ScribblePointerMode.all : ScribblePointerMode.penOnly,
      ),
      tooltip:
          "Switch drawing mode to " + (penMode ? "all pointers" : "pen only"),
      child: AnimatedSwitcher(
        duration: kThemeAnimationDuration,
        child: !penMode
            ? const Icon(
                Icons.touch_app,
                key: ValueKey(true),
              )
            : const Icon(
                Icons.do_not_touch,
                key: ValueKey(false),
              ),
      ),
    );
  }

  Widget _buildEraserButton(BuildContext context, {required bool isSelected}) {
    return Padding(
      padding: const EdgeInsets.all(4),
      child: FloatingActionButton.small(
        tooltip: "Erase",
        backgroundColor: const Color(0xFFF7FBFF),
        elevation: isSelected ? 10 : 2,
        shape: !isSelected
            ? const CircleBorder()
            : RoundedRectangleBorder(
                borderRadius: BorderRadius.circular(8),
              ),
        child: const Icon(Icons.remove, color: Colors.blueGrey),
        onPressed: notifier.setEraser,
      ),
    );
  }

  Widget _buildColorButton(
    BuildContext context, {
    required Color color,
    required ScribbleState state,
  }) {
    final isSelected = state is Drawing && state.selectedColor == color.value;
    return Padding(
      padding: const EdgeInsets.all(4),
      child: FloatingActionButton.small(
          backgroundColor: color,
          elevation: isSelected ? 10 : 2,
          shape: !isSelected
              ? const CircleBorder()
              : RoundedRectangleBorder(
                  borderRadius: BorderRadius.circular(8),
                ),
          child: Container(),
          onPressed: () => notifier.setColor(color)),
    );
  }

  Widget _buildUndoButton(
    BuildContext context,
  ) {
    return FloatingActionButton.small(
      tooltip: "Undo",
      onPressed: notifier.canUndo ? notifier.undo : null,
      disabledElevation: 0,
      backgroundColor: notifier.canUndo ? Colors.blueGrey : Colors.grey,
      child: const Icon(
        Icons.undo_rounded,
        color: Colors.white,
      ),
    );
  }

  Widget _buildRedoButton(
    BuildContext context,
  ) {
    return FloatingActionButton.small(
      tooltip: "Redo",
      onPressed: notifier.canRedo ? notifier.redo : null,
      disabledElevation: 0,
      backgroundColor: notifier.canRedo ? Colors.blueGrey : Colors.grey,
      child: const Icon(
        Icons.redo_rounded,
        color: Colors.white,
      ),
    );
  }

  Widget _buildClearButton(BuildContext context) {
    return FloatingActionButton.small(
      tooltip: "Clear",
      onPressed: notifier.clear,
      disabledElevation: 0,
      backgroundColor: Colors.blueGrey,
      child: const Icon(Icons.clear),
    );
  }
}
